==> distfun-0.3-1-src.zip: distfun/src/cdflib/brcomp.c <==
/* brcomp.f -- translated by f2c (version 20100827).
You must link the resulting object file with libf2c:
on Microsoft Windows system, link with libf2c.lib;
on Linux or Unix systems, link with .../path/to/libf2c.a -lm
or, if you install libf2c.a in a standard place, with -lf2c -lm
-- in that order, at the end of the command line, as in
cc *.o -lf2c -lm
Source for libf2c is in /netlib/f2c/libf2c.zip, e.g.,

http://www.netlib.org/f2c/libf2c.zip
*/

#include "cdflib.h"

double brcomp_(double *a, double *b, double *x, double *y)
{
	/* Initialized data */

	static double const__ = .398942280401433;

	/* System generated locals */
	int i__1;
	double ret_val, d__1;

	/* Builtin functions */
	double log(double), exp(double), sqrt(double);

	/* Local variables */
	static double c__, e, h__;
	static int i__, n;
	static double t, u, v, z__, a0, b0, x0, y0, apb, lnx, lny;
	static double lambda;

	/* ----------------------------------------------------------------------- */
	/*               EVALUATION OF X**A*Y**B/BETA(A,B) */
	/* ----------------------------------------------------------------------- */
	/*     .. Scalar Arguments .. */
	/*     .. */
	/*     .. Local Scalars .. */
	/*     .. */

	/*     .. */
	/*     .. Intrinsic Functions .. */
	/*     .. */
	/*     .. Data statements .. */
	/* ----------------- */
	/*     CONST = 1/SQRT(2*PI) */
	/* ----------------- */
	/*     .. */
	/*     .. Executable Statements .. */

	ret_val = 0.;
	if (*x == 0. || *y == 0.) {
		return ret_val;
	}
	a0 = min(*a,*b);
	if (a0 >= 8.) {
		goto L130;
	}

	if (*x > .375) {
		goto L10;
	}
	lnx = log(*x);
	d__1 = -(*x);
	lny = alnrel_(&d__1);
	goto L30;
L10:
	if (*y > .375) {
		goto L20;
	}
	d__1 = -(*y);
	lnx = alnrel_(&d__1);
	lny = log(*y);
	goto L30;
L20:
	lnx = log(*x);
	lny = log(*y);

L30:
	z__ = *a * lnx + *b * lny;
	if (a0 < 1.) {
		goto L40;
	}
	z__ -= betaln_(a, b);
	ret_val = exp(z__);
	return ret_val;
	/* ----------------------------------------------------------------------- */
	/*              PROCEDURE FOR A .LT. 1 OR B .LT. 1 */
	/* ----------------------------------------------------------------------- */
L40:
	b0 = max(*a,*b);
	if (b0 >= 8.) {
		goto L120;
	}
	if (b0 > 1.) {
		goto L70;
	}

	/*                   ALGORITHM FOR B0 .LE. 1 */

	ret_val = exp(z__);
	if (ret_val == 0.) {
		return ret_val;
	}

	apb = *a + *b;
	if (apb > 1.) {
		goto L50;
	}
	z__ = gam1_(&apb) + 1.;
	goto L60;
L50:
	u = *a + *b - 1.;
	z__ = (gam1_(&u) + 1.) / apb;

L60:
	c__ = (gam1_(a) + 1.) * (gam1_(b) + 1.) / z__;
	ret_val = ret_val * (a0 * c__) / (a0 / b0 + 1.);
	return ret_val;

	/*                ALGORITHM FOR 1 .LT. B0 .LT. 8 */

L70:
	u = gamln1_(&a0);
	n = (int) (b0 - 1.);
	if (n < 1) {
		goto L90;
	}
	c__ = 1.;
	i__1 = n;
	for (i__ = 1; i__ <= i__1; ++i__) {
		b0 += -1.;
		c__ *= b0 / (a0 + b0);
		/* L80: */
	}
	u = log(c__) + u;

L90:
	z__ -= u;
	b0 += -1.;
	apb = a0 + b0;
	if (apb > 1.) {
		goto L100;
	}
	t = gam1_(&apb) + 1.;
	goto L110;
L100:
	u = a0 + b0 - 1.;
	t = (gam1_(&u) + 1.) / apb;
L110:
	ret_val = a0 * exp(z__) * (gam1_(&b0) + 1.) / t;
	return ret_val;

	/*                   ALGORITHM FOR B0 .GE. 8 */

L120:
	u = gamln1_(&a0) + algdiv_(&a0, &b0);
	ret_val = a0 * exp(z__ - u);
	return ret_val;
	/* ----------------------------------------------------------------------- */
	/*              PROCEDURE FOR A .GE. 8 AND B .GE. 8 */
	/* ----------------------------------------------------------------------- */
L130:
	if (*a > *b) {
		goto L140;
	}
	h__ = *a / *b;
	x0 = h__ / (h__ + 1.);
	y0 = 1. / (h__ + 1.);
	lambda = *a - (*a + *b) * *x;
	goto L150;
L140:
	h__ = *b / *a;
	x0 = 1. / (h__ + 1.);
	y0 = h__ / (h__ + 1.);
	lambda = (*a + *b) * *y - *b;

L150:
	e = -lambda / *a;
	if (fabs(e) > .6) {
		goto L160;
	}
	u = rlog1_(&e);
	goto L170;
L160:
	u = e - log(*x / x0);

L170:
	e = lambda / *b;
	if (fabs(e) > .6) {
		goto L180;
	}
	v = rlog1_(&e);
	goto L190;
L180:
	v = e - log(*y / y0);

L190:
	z__ = exp(-(*a * u + *b * v));
	ret_val = const__ * sqrt(*b * x0) * z__ * exp(-bcorr_(a, b));
	return ret_val;
} /* brcomp_ */

==> distfun-0.5-1-src.zip: distfun/src/cdflib/brcomp.c <==
/* brcomp.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 

double cdflib_brcomp(double a, double b, double x, double y)
{
	/* Initialized data */

	static double const__ = .398942280401433;

	/* System generated locals */
	int i__1;
	double ret_val, d__1;

	/* Local variables */
	static double c__, e, h__;
	static int i__, n;
	static double t, u, v, z__, a0, b0, x0, y0, apb, lnx, lny;
	static double lambda;

	/* ----------------------------------------------------------------------- */
	/*               EVALUATION OF X**A*Y**B/BETA(A,B) */
	/* ----------------------------------------------------------------------- */

	/* ----------------- */
	/*     CONST = 1/SQRT(2*PI) */
	/* ----------------- */

	ret_val = 0.;
	if (x == 0. || y == 0.) {
		return ret_val;
	}
	a0 = min(a,b);
	if (a0 >= 8.) {
		goto L130;
	}

	if (x > .375) {
		goto L10;
	}
	lnx = log(x);
	d__1 = -(x);
	lny = cdflib_log1p(d__1);
	goto L30;
L10:
	if (y > .375) {
		goto L20;
	}
	d__1 = -(y);
	lnx = cdflib_log1p(d__1);
	lny = log(y);
	goto L30;
L20:
	lnx = log(x);
	lny = log(y);

L30:
	z__ = a * lnx + b * lny;
	if (a0 < 1.) {
		goto L40;
	}
	z__ -= cdflib_betaln(a, b);
	ret_val = exp(z__);
	return ret_val;
	/* ----------------------------------------------------------------------- */
	/*              PROCEDURE FOR A .LT. 1 OR B .LT. 1 */
	/* ----------------------------------------------------------------------- */
L40:
	b0 = max(a,b);
	if (b0 >= 8.) {
		goto L120;
	}
	if (b0 > 1.) {
		goto L70;
	}

	/*                   ALGORITHM FOR B0 .LE. 1 */

	ret_val = exp(z__);
	if (ret_val == 0.) {
		return ret_val;
	}

	apb = a + b;
	if (apb > 1.) {
		goto L50;
	}
	z__ = cdflib_gam1(apb) + 1.;
	goto L60;
L50:
	u = a + b - 1.;
	z__ = (cdflib_gam1(u) + 1.) / apb;

L60:
	c__ = (cdflib_gam1(a) + 1.) * (cdflib_gam1(b) + 1.) / z__;
	ret_val = ret_val * (a0 * c__) / (a0 / b0 + 1.);
	return ret_val;

	/*                ALGORITHM FOR 1 .LT. B0 .LT. 8 */

L70:
	u = cdflib_gamln1(a0);
	n = (int) (b0 - 1.);
	if (n < 1) {
		goto L90;
	}
	c__ = 1.;
	i__1 = n;
	for (i__ = 1; i__ <= i__1; ++i__) {
		b0 += -1.;
		c__ *= b0 / (a0 + b0);
		/* L80: */
	}
	u = log(c__) + u;

L90:
	z__ -= u;
	b0 += -1.;
	apb = a0 + b0;
	if (apb > 1.) {
		goto L100;
	}
	t = cdflib_gam1(apb) + 1.;
	goto L110;
L100:
	u = a0 + b0 - 1.;
	t = (cdflib_gam1(u) + 1.) / apb;
L110:
	ret_val = a0 * exp(z__) * (cdflib_gam1(b0) + 1.) / t;
	return ret_val;

	/*                   ALGORITHM FOR B0 .GE. 8 */

L120:
	u = cdflib_gamln1(a0) + cdflib_algdiv(a0, b0);
	ret_val = a0 * exp(z__ - u);
	return ret_val;
	/* ----------------------------------------------------------------------- */
	/*              PROCEDURE FOR A .GE. 8 AND B .GE. 8 */
	/* ----------------------------------------------------------------------- */
L130:
	if (a > b) {
		goto L140;
	}
	h__ = a / b;
	x0 = h__ / (h__ + 1.);
	y0 = 1. / (h__ + 1.);
	lambda = a - (a + b) * x;
	goto L150;
L140:
	h__ = b / a;
	x0 = 1. / (h__ + 1.);
	y0 = h__ / (h__ + 1.);
	lambda = (a + b) * y - b;

L150:
	e = -lambda / a;
	if (fabs(e) > .6) {
		goto L160;
	}
	u = cdflib_rlog1(e);
	goto L170;
L160:
	u = e - log(x / x0);

L170:
	e = lambda / b;
	if (fabs(e) > .6) {
		goto L180;
	}
	v = cdflib_rlog1(e);
	goto L190;
L180:
	v = e - log(y / y0);

L190:
	z__ = exp(-(a * u + b * v));
	ret_val = const__ * sqrt(b * x0) * z__ * exp(-cdflib_bcorr(a, b));
	return ret_val;
}
